#include <QHBoxLayout>
#include <QFileDialog>
#include <QPushButton>
#include <QApplication>
#include <QStyle>
#include <QPainter>
#include <QImageReader>

#include "custom-icon-select.h"

CustomIconSelect::CustomIconSelect(const QList<int> &icon_sizes, const QString &pix_path,  const QString &local_data_path, QWidget *parent):
    QWidget(parent)
{
    sizes = icon_sizes;
    orig_path = pix_path;
    custom_path = local_data_path;

    QHBoxLayout *l = new QHBoxLayout(this);

    min_size = 64;
    max_size = 16;
    foreach(int size, sizes)
    {
	QPixmap pix((custom_path+"/transmission-%1.png").arg(size));
	if( pix.isNull() )
	    pix.load((orig_path+"/transmission-%1.png").arg(size));
	if( pix.isNull() )
	    continue;
	if( size <= min_size )
	{
	    min_size = size;
	    pix_min = pix;
	}
	if( size >= max_size && size <= 64 )
	{
	    max_size = size;
	    pix_max = pix;
	}
    }

    pix_min_lbl = new QLabel(this);
    pix_min_lbl->setPixmap(pix_min);
    pix_max_lbl = new QLabel(this);
    pix_max_lbl->setPixmap(pix_max);
    l->addWidget(pix_min_lbl);
    l->addWidget(pix_max_lbl);

    QPushButton *filebtn = new QPushButton(tr("Customize"), this);
    filebtn->setIcon(QIcon::fromTheme("open-folder", QApplication::style()->standardPixmap(QStyle::SP_DirOpenIcon)));
    l->addWidget(filebtn);
    connect(filebtn, SIGNAL(clicked()), this, SLOT(selectPixmap()));

    QPushButton *clearbtn = new QPushButton(tr("Reset"), this);
    clearbtn->setIcon(QIcon::fromTheme("edit-clear", QApplication::style()->standardPixmap(QStyle::SP_DialogCloseButton)));
    l->addWidget(clearbtn);
    connect(clearbtn, SIGNAL(clicked()), this, SLOT(clearPixmap()));
}

CustomIconSelect::~CustomIconSelect()
{}

void CustomIconSelect::pixmapSelected(const QString &path)
{
    QPixmap pxmin((orig_path+"/transmission-%1.png").arg(min_size));
    QPixmap pxmax((orig_path+"/transmission-%1.png").arg(max_size));
    QFileInfo finfo(path);
    if( finfo.exists() && finfo.isFile() && finfo.isReadable() )
    {
	QPixmap overlay(path);
	applyPixmap(pxmin, overlay);
	applyPixmap(pxmax, overlay);
	overlay_path = path;
    }
    else
	overlay_path.clear();
    pix_min = pxmin;
    pix_max = pxmax;
    pix_min_lbl->setPixmap(pix_min);
    pix_max_lbl->setPixmap(pix_max);
}

void CustomIconSelect::selectPixmap()
{
    QString filter;
    foreach(QByteArray format, QImageReader::supportedImageFormats())
	filter.append(" *."+format.toLower());
    QFileDialog dialog(this, tr("Select overlaying picture"), last_directory, tr("Images %1").arg("("+filter+")"));
    dialog.setFileMode( QFileDialog::ExistingFile );
    dialog.setAcceptMode( QFileDialog::AcceptOpen );
    dialog.setOption( QFileDialog::ReadOnly, true );
    connect(&dialog, SIGNAL(fileSelected(const QString&)), this, SLOT(pixmapSelected(const QString&)));
    dialog.exec();
    last_directory = dialog.directory().path();
}

void CustomIconSelect::clearPixmap()
{
    pixmapSelected("");
}

void CustomIconSelect::applyPixmap(QPixmap &pix, const QPixmap &overlay)
{
    QPixmap over(overlay);
    if( over.width() > pix.width()/1.375 || over.height() > pix.height()/1.375 )
	over = over.scaled(pix.size()/1.375, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    QPainter p(&pix);
    p.drawPixmap(pix.width()-over.width(),
	pix.height()-over.height(),
	over);
}

void CustomIconSelect::save()
{
    QDir(custom_path).mkpath(custom_path);
    foreach(int size, sizes)
    {
	QString orig_pix_path((orig_path+"/transmission-%1.png").arg(size));
	QString custom_pix_path((custom_path+"/transmission-%1.png").arg(size));

	QPixmap orig_pix(orig_pix_path);
	QPixmap overlay_pix;
	if( !overlay_path.isEmpty() )
	    overlay_pix.load(overlay_path);
	if( overlay_path.isEmpty() || orig_pix.isNull() || overlay_pix.isNull() )
	{
	    QFile custom_pix_file(custom_pix_path);
	    if( custom_pix_file.exists() )
		custom_pix_file.remove();
	}
	else
	{
	    applyPixmap(orig_pix, overlay_pix);
	    orig_pix.save(custom_pix_path);
	}
    }
}
